/*
 -------------------------------------------------------------------------
 Copyright (c) 2002-2006, XIMETA, Inc., FREMONT, CA, USA.
 All rights reserved.

 LICENSE TERMS

 The free distribution and use of this software in both source and binary 
 form is allowed (with or without changes) provided that:

   1. distributions of this source code include the above copyright 
      notice, this list of conditions and the following disclaimer;

   2. distributions in binary form include the above copyright
      notice, this list of conditions and the following disclaimer
      in the documentation and/or other associated materials;

   3. the copyright holder's name is not used to endorse products 
      built using this software without specific written permission. 
      
 ALTERNATIVELY, provided that this notice is retained in full, this product
 may be distributed under the terms of the GNU General Public License (GPL),
 in which case the provisions of the GPL apply INSTEAD OF those given above.
 
 DISCLAIMER

 This software is provided 'as is' with no explcit or implied warranties
 in respect of any properties, including, but not limited to, correctness 
 and fitness for purpose.
 -------------------------------------------------------------------------
*/
#include "linux.h" 
#if !LINUX_VERSION_25_ABOVE
#include <linux/module.h> // THIS_MODULE
#include <linux/slab.h> /* kmalloc, kfree */
#include <ndasuser/ndasuser.h>
#include <ndasuser/io.h>
#include <sal/debug.h>

#include "block.h"
#include "ops.h"
#include "procfs.h"

#ifdef NDAS_MSHARE

#include <ndasuser/mediaop.h>


int get_req_size(struct request * req, unsigned int max_req_size)
{
	struct buffer_head * bh;
	unsigned int len = 0, slen = 0;
	bh = req->bh;
	while(bh) {
		slen = bh->b_size;
		if(len + slen > max_req_size) break;
		len += slen;
		bh = bh->b_reqnext;
	}

	return len;
}

////////////////////////////////////////////////////////////////////
//
//
//		encryp / decrypt function
//
//
//
////////////////////////////////////////////////////////////////////

void encrypt_req( int slot, int partnum, struct request * req)
{
	struct buffer_head * bh;
	void * key = NULL;

	key = ndas_getkey(slot, partnum);
	if(key == NULL) return;
	
	bh = req->bh;
	while(bh) {
		ndas_encrypt(key, bh->b_data, bh->b_data, bh->b_size);
		bh = bh->b_reqnext;
	}
	return ;
}


void decrypt_req(int slot, int partnum, struct request * req)
{
	struct buffer_head * bh;
	void * key = NULL;
	
	key = ndas_getkey(slot, partnum);
	if(key == NULL) return;

	bh = req->bh;
	while(bh) {
		ndas_decrypt(key, bh->b_data, bh->b_data, bh->b_size);
		bh = bh->b_reqnext;
	}
	return ;
}




struct nbio* lio_alloc_with_offset(struct request *req, unsigned int offset, unsigned int req_size) 
{
	int len = 0;
	struct buffer_head *bh;
	int slot = SLOT_R(req);
	struct nbio *lio = nblk_mem_alloc();
			/* size: NBIO_SIZE(req->nr_phys_segments) */
	unsigned int max_sectors = NDAS_GET_SLOT_DEV(slot)->info.max_sectors_per_request;

	unsigned int total_chk_size = 0;
	
	unsigned int mg = 0;
	dbgl_blk(8, "nr_segments=%d", req->nr_segments);

	if ( !lio ) return NULL;
	
	lio->req = req;
	lio->nr_blocks = 0;
	lio->dev = NDAS_GET_SLOT_DEV(slot);

	if(offset > 0){
		unsigned int bSearch = 0;
		for(bh = req->bh; bh; bh = bh->b_reqnext)
		{
			if(bSearch != 1){
				if( (total_chk_size + bh->b_size) > offset ) {
					mg = total_chk_size + bh->b_size - offset;
				}else{
					total_chk_size += bh->b_size;
					continue;
				}

				if(bSearch != 1){
					bSearch  = 1;
					if(len + mg > max_sectors * 1024) {
						dbgl_blk(1, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024 , lio->nr_blocks);
						break;
					}

					if(len + mg > req_size) {
						dbgl_blk(1, "ed len=%d/req_size%d nr_uio=%d", len, req_size , lio->nr_blocks);
						break;
					}
					
					lio->blocks[lio->nr_blocks].len = mg;
					lio->blocks[lio->nr_blocks].ptr = (bh->b_data + mg);
					lio->nr_blocks++;
					len += mg;
					if ( bh->b_reqnext == NULL ) 
						dbgl_blk(8, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024,lio->nr_blocks );
					}
			}else {
					if(len + bh->b_size > max_sectors*1024) {
						dbgl_blk(1, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024 , lio->nr_blocks);
						break;
					}

					if(len + mg > req_size) {
						dbgl_blk(1, "ed len=%d/req_size%d nr_uio=%d", len, req_size , lio->nr_blocks);
						break;
					}
					
					if ( NBIO_ADJACENT(lio,bh->b_data) )
					{
						lio->blocks[lio->nr_blocks - 1].len += bh->b_size;
					} else {
						lio->blocks[lio->nr_blocks].len = bh->b_size;
						lio->blocks[lio->nr_blocks].ptr = bh->b_data;
						lio->nr_blocks++;
					}
					len += bh->b_size;
					if ( bh->b_reqnext == NULL ) 
						dbgl_blk(8, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024,lio->nr_blocks );

			}
		}
	}else{
	
		for (bh = req->bh; bh; bh = bh->b_reqnext) 
		{	
			if(len + bh->b_size > max_sectors*1024) {
				dbgl_blk(1, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024 , lio->nr_blocks);
				break;
			}

			if(len + mg > req_size) {
				dbgl_blk(1, "ed len=%d/req_size%d nr_uio=%d", len, req_size , lio->nr_blocks);
				break;
			}
			
			if ( NBIO_ADJACENT(lio,bh->b_data) )
			{
				lio->blocks[lio->nr_blocks - 1].len += bh->b_size;
			} else {
				lio->blocks[lio->nr_blocks].len = bh->b_size;
				lio->blocks[lio->nr_blocks].ptr = bh->b_data;
				lio->nr_blocks++;
			}
			len += bh->b_size;
			if ( bh->b_reqnext == NULL ) 
				dbgl_blk(8, "ed len=%d/%d nr_uio=%d", len, max_sectors*1024,lio->nr_blocks );
		}
	}
	lio->nr_sectors = len >> 9;;
	return lio;
}



ndas_error_t nblk_handle_juke_io(struct request *req) 
{
	ndas_error_t err = NDAS_OK;
	ndas_io_request request;
	int slot = SLOT_R(req);
	int partnum =PARTITION(req->rq_dev); 
	struct ndas_slot *sd = NDAS_GET_SLOT_DEV(slot);
	struct nbio *lio = NULL;
	

	unsigned int	req_sectors = 0;
	unsigned int	start_sector = 0;	
	unsigned int 	org_start_sector = 0,  org_req_sectors =  0;
	unsigned int 	chk_start_sector = 0, chk_req_sectors = 0;
	unsigned int 	max_req_size = (NDAS_GET_SLOT_DEV(slot)->info.max_sectors_per_request * 1024) ;
	unsigned int 	processed_sector_count = 0;

#if	(LINUX_VERSION_CODE <= KERNEL_VERSION(2,4,22))
	if(sd->info.mode != NDAS_DISK_MODE_MEDIAJUKE  || partnum == 0){
#else
	if(sd->info.mode != NDAS_DISK_MODE_MEDIAJUKE ){
#endif
		return nblk_handle_io(req);
	}


	if(!ndas_IsDiscSet(slot,partnum))
	{
		if(partnum == 0){
			printk("end reqest by ilgu\n");
            		while(end_that_request_first(req, 1, DEVICE_NAME));
            			end_that_request_last(req);
			return 0;
		}
		printk("Disc is not set!\n");
		return NDAS_ERROR_JUKE_DISC_NOT_SET;
	}
	

	chk_start_sector =org_start_sector = req->sector ;
	chk_req_sectors =org_req_sectors =  (get_req_size(req, max_req_size)/512);

	//printk("call is not encrypt!\n");
	if(REQ_DIR(req) == WRITE)  encrypt_req(slot, partnum,req);
	
	
	while(chk_req_sectors > 0){
			// step 1  : address translation
		if(!ndas_TranslateAddr(slot,partnum,chk_start_sector, chk_req_sectors, &start_sector, &req_sectors))
		{
			printk("can't translate address org_start(%d), org_sec(%d)\n", chk_start_sector, chk_req_sectors);
			return NDAS_ERROR_JUKE_DISC_ADDR_NOT_FOUND;
		}
		//printk("call is ndas_TranslateAddrorg_start(%d), org_sec(%d), req_start(%d), req_sec(%d)\n", chk_start_sector, chk_req_sectors, start_sector, req_sectors);
		// step 2 : alloc linux io req
		lio = lio_alloc_with_offset(req, (processed_sector_count * 512), (req_sectors * 512)); //  req, offset, req_size
	
		if ( !lio ) {
			return NDAS_ERROR_OUT_OF_MEMORY;
		}

		if (!sd->enabled) {
			kmem_cache_free(nblk_kmem_cache ,lio);
			printk("Requesting to disabled device\n");
			return NDAS_ERROR_NOT_CONNECTED;
		}

	
		//printk("IOR:%d\n", lio->nr_sectors);
		request.uio = lio->blocks;
		request.buf = NULL;
		request.nr_uio = lio->nr_blocks;
		request.num_sec = lio->nr_sectors;
		request.start_sec = start_sector;
		request.done = NULL; // Synchrounos IO : sjcho temp
		
		dbgl_blk(8, 
			"req(%p)=bh(%p)={cmd=%s, nr_uio=%d, nr_sectors=%d, start sector=%Lu}",
			req,
			req->bh, 
			(req->cmd == READ || req->cmd == READA) ?"READ":"WRITE",
			request.nr_uio, 
			request.num_sec,
			request.start_sec);

		switch ( REQ_DIR(req)) {
		case READ:
		case READA:
			//printk("ndas_read!\n");
			err = ndas_read(slot,&request);
			break;
		case WRITE:
			err = ndas_write(slot,&request);
			break;
		default:
			err = NDAS_ERROR_INVALID_OPERATION;
			printk("ndas: operation %d is not supported.\n", REQ_DIR(req));
		}
		//printk("after_call!\n");
		processed_sector_count += lio->nr_sectors;
		chk_start_sector += lio->nr_sectors;
		chk_req_sectors  -=lio->nr_sectors;

		if(processed_sector_count == org_req_sectors) break;
		else kmem_cache_free(nblk_kmem_cache ,lio);
	}	
	
	if(( REQ_DIR(req) == READ) || ( REQ_DIR(req) == READA))  decrypt_req(slot, partnum,req);
	

	nblk_end_io(slot, err, lio);

	return err;	
}
#endif /*#ifdef NDAS_MSHARE */
#endif /*#if !LINUX_VERSION_25_ABOVE */
